# 
# Janosik - Wieloplatformowa odpowiednik programu Patnik 
#
# Copyright (C) 2002, 2003 Janosik Development Team
# 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

"""Modu weryfikujcy dokumenty ZUS-owskie.

	Wszystkie pola wszystkich dokumantw maj tu okrelone 
	funkcje sprawdzajce ich poprawno. W module znajduj si
	te definicje tych funkcji. (Te wszystkie dokumenty, to troch 
	na wyrost, ale tak _powinno_ by. :)"""

import re
import calendar
import kody

def _dni_w_miesiacu(miesiac, przestepny):
	if miesiac == 1:
		return 31
	elif miesiac == 2 and not przestepny:
		return 28
	elif miesiac == 2 and przestepny:
		return 29
	elif miesiac == 3:
		return 31
	elif miesiac == 4:
		return 30
	elif miesiac == 5:
		return 31
	elif miesiac == 6:
		return 30
	elif miesiac == 7:
		return 31
	elif miesiac == 8:
		return 31
	elif miesiac == 9:
		return 30
	elif miesiac == 10:
		return 31
	elif miesiac == 11:
		return 30
	elif miesiac == 12:
		return 31
	else:
		return 0

def data_ddmm(wart, slownik):
	if len(wart) != 4:
		return 0
	if wart.isspace():
		return 1
	if not wart.isdigit():
		return 0
	dzien = int(wart[:2])
	miesiac = int(wart[2:])
	if miesiac < 1 or miesiac > 12:
		return 0
	if dzien < 1 or dzien > _dni_w_miesiacu(miesiac, 1):
		return 0
	return 1

def data_ddmmrrrr(wart, slownik):
	if len(wart) != 8:
		return 0
	if wart.isspace():
		return 1
	if not wart.isdigit():
		return 0
	dzien = int(wart[:2])
	miesiac = int(wart[2:4])
	rok = int(wart[4:])
	if miesiac < 1 or miesiac > 12:
		return 0
	if dzien < 1 or dzien > _dni_w_miesiacu(miesiac, calendar.isleap(rok)):
		return 0
	return 1

def data_mmrrrr(wart, slownik):
	if len(wart) != 6:
		return 0
	if wart.isspace():
		return 1
	if not wart.isdigit():
		return 0
	miesiac = int(wart[:2])
	rok = int(wart[2:])
	if miesiac < 1 or miesiac > 12:
		return 0
	return 1

def _zamien(data):
	return data[4:8] + data[2:4] + data[0:2]

def okres(wart, slownik):
	if len(wart) != 16:
		return 0
	if wart.isspace():
		return 1
	wart = wart.rstrip()
	if len(wart) == 8:
		return data_ddmmrrrr(wart, slownik)
	else:
		if not data_ddmmrrrr(wart[:8], slownik) or not data_ddmmrrrr(wart[8:], slownik):
			return 0
		return _zamien(wart[:8]) < _zamien(wart[8:])
	
def dowolne(wart, slownik, dlugosc):
	return len(wart) == dlugosc

def lancuch(wart, slownik, dlugosc):
	if len(wart) != dlugosc:
		return 0
	if wart.isspace():
		return 1
	wart = wart.rstrip()
	return re.match('^[A-ZʣӦ]+$', wart)

def liczba(wart, slownik, dlugosc):
	if len(wart) != dlugosc:
		return 0
	if wart.isspace():
		return 1
	return wart.isdigit()

def liczba_dop(wart, slownik, dlugosc):
	if len(wart) != dlugosc:
		return 0
	if wart.isspace():
		return 1
	wart = wart.strip()
	return wart.isdigit()

def puste(wart, slownik, dlugosc):
	return wart.isspace() and len(wart) == dlugosc

def imie_22(wart, slownik):
	return lancuch(wart, slownik, 22)

def obywatelstwo_22(wart, slownik):
	return lancuch(wart, slownik, 22)

def nazwisko_31(wart, slownik):
	if len(wart) != 31:
		return 0
	if wart.isspace():
		return 1
	wart = wart.rstrip()
	return re.match('^[A-ZʣӦ]+(-[A-ZʣӦ]+)?$', wart)
	
def nazwa(wart, slownik, dlugosc):
	if len(wart) != dlugosc:
		return 0
	if wart.isspace():
		return 1
	return re.match('^[0-9A-ZʣӦ\-,./"_ ]+$', wart)

def tak_nie(wart, slownik):
	return wart == 'X' or wart == ' '

def lista(wart, slownik, lista):
	if wart.isspace():
		return 1
	return wart in map(lambda t: t[0], lista)

def lista_3(wart, slownik, lista1, poz1, lista2, poz2, lista3):
	if wart.isspace():
		return 1
	return lista(wart[:poz1], slownik, lista1) and lista(wart[poz1:poz2], slownik, lista2) and lista(wart[poz2:], slownik, lista3)
	
def _suma(wart, wagi):
	cyfry = map(int, wart)
	return reduce(lambda a, b: a + b, map(lambda a, b: a * b, wagi, cyfry))

def _nip_suma(wart):
	wagi = [6, 5, 7, 2, 3, 4, 5, 6, 7]
	return _suma(wart, wagi) % 11 % 10 

def nip(wart, slownik):
	if not liczba(wart, slownik, 10): 
		return 0
	if wart.isspace():
		return 1
	return int(wart[-1]) == _nip_suma(wart[:-1])
	
def _pesel_suma(wart):
	wagi = [1, 3, 7, 9, 1, 3, 7, 9, 1, 3]
	return (10 - _suma(wart, wagi) % 10) % 10

def pesel(wart, slownik):
	if not liczba(wart, slownik, 11): 
		return 0
	if wart.isspace():
		return 1
	return int(wart[-1]) == _pesel_suma(wart[:-1])
	
def _regon9_suma(wart):
	wagi = [8, 9, 2, 3, 4, 5, 6, 7]
	return _suma(wart, wagi) % 11 % 10

def _regon14_suma(wart):
	wagi = [2, 4, 8, 5, 0, 9, 7, 3, 6, 1, 2, 4, 8]
	return _suma(wart, wagi) % 11 % 10

def regon(wart, slownik):
	if len(wart) != 14:
		return 0
	if wart.isspace():
		return 1
	wart = wart.rstrip()
	if not wart.isdigit():
		return 0
	if len(wart) == 9:
		return int(wart[-1]) == _regon9_suma(wart[:-1])
	if len(wart) == 14:
		return int(wart[-1]) == _regon14_suma(wart[:-1])

def seria_numer_9(wart, slownik):
	if len(wart) != 9:
		return 0
	if wart.isspace():
		return 1
	wart = wart.rstrip()
	return re.match('^[0-9A-Z]+$', wart)

def telefon_12(wart, slownik):
	if len(wart) != 12:
		return 0
	if wart.isspace():
		return 1
	wart = wart.rstrip()
	return re.match('^[0-9\- ]+$', wart)

def adres_30(wart, slownik):
	if len(wart) != 30:
		return 0
	if wart.isspace():
		return 1
	wart = wart.rstrip()
	return re.match('^[0-9A-Z\-._]+@[0-9A-Z\-._]+$', wart)

def plec(wart, slownik, nazwa):
	plec_pesel = slownik[nazwa][-2] in ('1', '3', '5', '7', '9')
	plec_wart = wart == 'M'
	return plec_wart == plec_pesel

def konto_36(wart, slownik):
	if len(wart) != 36:
		return 0
	if wart.isspace():
		return 1
	wart = wart.rstrip()
	return re.match('^[0-9\-]+$', wart)

def urodziny(wart, slownik, nazwa):
	urodziny_pesel = slownik[nazwa][:6]
	urodziny_wart = wart[-2:] + wart[2:4] + wart[:2]
	return urodziny_wart == urodziny_pesel

	
dlugosc_komunikat = 'Za dugo pola.'
tak_nie_komunikat = 'Wstaw X lub spacj.'
data_ddmmrrrr_komunikat = 'Wstaw poprawn date w postaci DDMMRRRR.'
okres_komunikat = 'Wstaw poprawny okres czasu w postaci DDMMRRRRDDMMRRRR.'
nip_komunikat = 'Wstaw poprawny numer NIP.'
regon_komunikat = 'Wstaw poprawny numer REGON.'
pesel_komunikat = 'Wstaw poprawny numer PESEL.'
puste_komunikat = 'Zostaw niewypenione.'
lista_komunikat = 'Wstaw warto z podanej listy.'
seria_numer_komunikat = 'Wstaw serie i numer dokumentu.'
nazwa_komunikat = 'Wstaw poprawn nazw.'
nazwisko_komunikat = 'Wstaw poprawne nazwisko.'
imie_komunikat = 'Wstaw poprawne imi.'
obywatelstwo_komunikat = 'Wstaw poprawne obywatelstwo.'
liczba_komunikat = 'Wstaw poprawn liczb.'
kod_komunikat = 'Wstaw poprwny kod pocztowy.'
numer_komunikat = 'Wstaw poprwny numer.'
adres_komunikat = 'Wstaw poprawny adres e-mail.'
plec_komunikat = 'Pe musi si zgadza z numerem PESEL'
urodziny_komunikat = 'Data urodzenia musi si zgadza z numerem PESEL'
konto_komunikat = 'Wstaw poprawny numer konta.'


# W sowniku trzymam sowniki indeksowane nazw pola, wartoci jest lista
# tuples zawierajca funkcje_sprawdzajc, komunikat_bdu, 
# list_dodatkowych_argumentw_funkcji_sprawdzajcej.

tablica_wer = {}

tablica_wer['zuszcza'] = {
	'dozcba_data_nadania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dozcba_numer_nalepki_r': [(dowolne, dlugosc_komunikat, [6])],
	'dipl_nip': [(nip, nip_komunikat, [])],
	'dipl_regon': [(regon, regon_komunikat, [])],
	'dipl_pesel': [(pesel, pesel_komunikat, [])],
	'dipl_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dipl_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dipl_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dipl_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dipl_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dipl_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dipl_pesel'])],
	'dipl_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'dau_pesel': [(pesel, pesel_komunikat, [])],
	'dau_nip': [(nip, nip_komunikat, [])],
	'dau_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dau_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dau_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dau_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dau_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dau_pesel'])],
	'dau_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dau_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dau_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'docrba_zgloszenie_nczr': [(lista, lista_komunikat, [kody.rodzaj_zgloszenia_czlonka_rodziny])],
	'docrba_data_uzyskania_uuz': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'docrba_pesel': [(pesel, pesel_komunikat, [])],
	'docrba_nip': [(nip, nip_komunikat, [])],
	'docrba_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'docrba_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'docrba_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'docrba_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'docrba_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'docrba_kod_pokrewienstwa': [(lista, lista_komunikat, [kody.stopien_pokrewienstwa_lub_powinowactwa])],
	'docrba_wylaczne_utrzymanie': [(tak_nie, tak_nie_komunikat, [])],
	'docrba_wspolne_gospodarstwo': [(tak_nie, tak_nie_komunikat, [])],
	'docrba_kod_niepelnosprawn': [(lista, lista_komunikat, [kody.tytul_ubezpieczenia_stopien])],
	'docrba_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'docrba_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'docrba_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'opl1_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])]
	}


tablica_wer['zuszcna'] = {
	'dozcba_data_nadania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dozcba_numer_nalepki_r': [(dowolne, dlugosc_komunikat, [6])],
	'dipl_nip': [(nip, nip_komunikat, [])],
	'dipl_regon': [(regon, regon_komunikat, [])],
	'dipl_pesel': [(pesel, pesel_komunikat, [])],
	'dipl_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dipl_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dipl_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dipl_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dipl_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dipl_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dipl_pesel'])],
	'dipl_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'dau_pesel': [(pesel, pesel_komunikat, [])],
	'dau_nip': [(nip, nip_komunikat, [])],
	'dau_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dau_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dau_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dau_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dau_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dau_pesel'])],
	'dau_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dau_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dau_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'docra_zgloszenie_nczr': [(lista, lista_komunikat, [kody.rodzaj_zgloszenia_czlonka_rodziny])],
	'docra_data_uzyskania_uuz': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'docra_pesel': [(pesel, pesel_komunikat, [])],
	'docra_nip': [(nip, nip_komunikat, [])],
	'docra_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'docra_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'docra_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'docra_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'docra_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'docra_kod_pokrewienstwa': [(lista, lista_komunikat, [kody.stopien_pokrewienstwa_lub_powinowactwa])],
	'docra_wylaczne_utrzymanie': [(tak_nie, tak_nie_komunikat, [])],
	'docra_wspolne_gospodarstwo': [(tak_nie, tak_nie_komunikat, [])],
	'docra_kod_niepelnosprawn': [(lista, lista_komunikat, [kody.tytul_ubezpieczenia_stopien])],
	'docra_kod': [(liczba, kod_komunikat, [5])], 
	'docra_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'docra_gmina': [(nazwa, nazwa_komunikat, [26])],
	'docra_ulica': [(nazwa, nazwa_komunikat, [30])],
	'docra_dom': [(nazwa, numer_komunikat, [7])],
	'docra_lokal': [(nazwa, numer_komunikat, [7])],
	'docra_telefon': [(telefon_12, numer_komunikat, [])],
	'docra_fax': [(telefon_12, numer_komunikat, [])],
	'docra_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'docra_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'docra_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'opl1_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])]
	}


tablica_wer['zuszfa'] = {
	'dozpf_zgloszenie_platnika_skladek': [(tak_nie, tak_nie_komunikat, [])],
	'dozpf_zgloszenie_korekty': [(lista, lista_komunikat, [kody.zgloszenie_zmiany_korekta_danych])],
	'dozpf_data_nadania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dozpf_numer_nalepki_r': [(dowolne, dlugosc_komunikat, [6])],
	'dipl_nip': [(nip, nip_komunikat, [])],
	'dipl_regon': [(regon, regon_komunikat, [])],
	'dipl_pesel': [(pesel, pesel_komunikat, [])],
	'dipl_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dipl_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dipl_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dipl_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dipl_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dipl_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dipl_pesel'])],
	'dipl_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'depl_imie_drugie': [(imie_22, imie_komunikat, [])],
	'depl_miejsce_urodzenia': [(nazwa, nazwa_komunikat, [26])],
	'depl_obywatelstwo': [(obywatelstwo_22, obywatelstwo_komunikat, [])],
	'ppdb_kod_rodzaju': [(lista, lista_komunikat, [kody.rodzaj_uprawnienia])],
	'ppdb_numer_uprawnienia': [(nazwa, numer_komunikat, [15])],
	'ppdb_data_wydania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'ppdb_data_rozpoczecia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dorb_numer': [(konto_36, konto_komunikat, [])],
	'dorb_inne_rachunki': [(tak_nie, tak_nie_komunikat, [])],
	'idop_status': [(tak_nie, tak_nie_komunikat, [])],
	'idop_data_otrzymania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'idop_data_utraty': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'idop_data_powstania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'idop_adres': [(tak_nie, tak_nie_komunikat, [])],
	'aspl_kod': [(liczba, kod_komunikat, [5])], 
	'aspl_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'aspl_gmina': [(nazwa, nazwa_komunikat, [26])],
	'aspl_ulica': [(nazwa, nazwa_komunikat, [30])],
	'aspl_dom': [(nazwa, numer_komunikat, [7])],
	'aspl_lokal': [(nazwa, numer_komunikat, [7])],
	'aspl_telefon': [(telefon_12, numer_komunikat, [])],
	'aspl_fax': [(telefon_12, numer_komunikat, [])],
	'aspl_email': [(adres_30, adres_komunikat, [])],
	'adkp_kod': [(liczba, kod_komunikat, [5])], 
	'adkp_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'adkp_ulica': [(nazwa, nazwa_komunikat, [30])],
	'adkp_dom': [(nazwa, numer_komunikat, [7])],
	'adkp_lokal': [(nazwa, numer_komunikat, [7])],
	'adkp_telefon_do_tt': [(telefon_12, numer_komunikat, [])],
	'adkp_skrytka': [(nazwa, numer_komunikat, [5])],
	'adkp_telefon': [(telefon_12, numer_komunikat, [])],
	'adkp_fax': [(telefon_12, numer_komunikat, [])],
	'adkp_email': [(adres_30, adres_komunikat, [])],
	'dobr_nip': [(nip, nip_komunikat, [])],
	'dobr_regon': [(regon, regon_komunikat, [])],
	'dobr_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dobr_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dobr_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dobr_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'opl2_liczba_zba': [(liczba_dop, liczba_komunikat, [3])],
	'opl2_liczba_zaa': [(liczba_dop, liczba_komunikat, [3])],
	'opl2_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])]
	}
	

tablica_wer['zuszipa'] = {
	'dozipa_zgloszenie_korekty_prawna': [(lista, lista_komunikat, [kody.zgloszenie_zmiany_korekta_danych])],
	'dozipa_zgloszenie_korekty_fizyczna': [(lista, lista_komunikat, [kody.zgloszenie_zmiany_korekta_danych])],
	'doziua_data_nadania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'doziua_numer_nalepki_r': [(dowolne, dlugosc_komunikat, [6])],
	'dipl_nip': [(nip, nip_komunikat, [])],
	'dipl_regon': [(regon, regon_komunikat, [])],
	'dipl_pesel': [(pesel, pesel_komunikat, [])],
	'dipl_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dipl_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dipl_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dipl_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dipl_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dipl_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dipl_pesel'])],
	'dipl_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'adip_nip': [(nip, nip_komunikat, [])],
	'adip_regon': [(regon, regon_komunikat, [])],
	'adip_pesel': [(pesel, pesel_komunikat, [])],
	'adip_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'adip_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'adip_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'adip_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'adip_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'adip_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['adip_pesel'])],
	'adip_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'adip_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'adip_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'opl1_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])]
	}


tablica_wer['zusziua'] = {
	'doziua_zgloszenie_korekty': [(lista, lista_komunikat, [kody.zgloszenie_zmiany_korekta_danych])],
	'doziua_data_nadania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'doziua_numer_nalepki_r': [(dowolne, dlugosc_komunikat, [6])],
	'dipl_nip': [(nip, nip_komunikat, [])],
	'dipl_regon': [(regon, regon_komunikat, [])],
	'dipl_pesel': [(pesel, pesel_komunikat, [])],
	'dipl_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dipl_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dipl_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dipl_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dipl_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dipl_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dipl_pesel'])],
	'dipl_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'diop_pesel': [(pesel, pesel_komunikat, [])],
	'diop_nip': [(nip, nip_komunikat, [])],
	'diop_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'diop_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'diop_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'diop_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'diop_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'diop_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'diop_numer_wer_danych_id': [(puste, dlugosc_komunikat, [16])],
	'diop_poziom_ufnosci_id': [(puste, dlugosc_komunikat, [2])],
	'adiu_pesel': [(pesel, pesel_komunikat, [])],
	'adiu_nip': [(nip, nip_komunikat, [])],
	'adiu_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'adiu_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'adiu_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'adiu_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'adiu_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'adiu_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'adiu_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'adiu_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'opl1_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])]
	}


tablica_wer['zuszua'] = {
	'dozua_zgloszenie_do_us_i_uz': [(tak_nie, tak_nie_komunikat, [])],
	'dozua_zgloszenie_do_us': [(tak_nie, tak_nie_komunikat, [])],
	'dozua_zgloszenie_korekty': [(lista, lista_komunikat, [kody.zgloszenie_zmiany_korekta_danych])],
	'dozua_data_nadania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dozua_numer_nalepki_r': [(dowolne, dlugosc_komunikat, [6])],
	'dipl_nip': [(nip, nip_komunikat, [])],
	'dipl_regon': [(regon, regon_komunikat, [])],
	'dipl_pesel': [(pesel, pesel_komunikat, [])],
	'dipl_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dipl_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dipl_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dipl_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dipl_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dipl_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dipl_pesel'])],
	'dipl_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'dau_pesel': [(pesel, pesel_komunikat, [])],
	'dau_nip': [(nip, nip_komunikat, [])],
	'dau_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dau_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dau_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dau_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dau_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dau_pesel'])],
	'dau_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dau_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dau_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'deoz_imie_drugie': [(imie_22, imie_komunikat, [])],
	'deoz_nazwisko_rodowe': [(nazwisko_31, nazwisko_komunikat, [])],
	'deoz_obywatelstwo': [(obywatelstwo_22, obywatelstwo_komunikat, [])],
	'deoz_plec': [(lista, lista_komunikat, [kody.oznaczenie_plci]),
			(plec, plec_komunikat, ['dau_pesel'])],
	'deoz_czy_ma_karte': [(tak_nie, tak_nie_komunikat, [])],
	'deoz_czy_czasowo': [(tak_nie, tak_nie_komunikat, [])],
	'tyub_kod': [(lista_3, lista_komunikat, 
			[kody.tytul_ubezpieczenia_podmiot,
			4, kody.tytul_ubezpieczenia_prawo,
			5, kody.tytul_ubezpieczenia_stopien])],
	'tyub_okres': [(okres, okres_komunikat, [])],
	'dowp_wymiar_etatu_l': [(liczba_dop, liczba_komunikat, [3])],
	'dowp_wymiar_etatu_m': [(liczba_dop, liczba_komunikat, [3])],
	'dowp_data_powstania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dowp_emerytalne': [(tak_nie, tak_nie_komunikat, [])],
	'dowp_rentowe': [(tak_nie, tak_nie_komunikat, [])],
	'dowp_chorobowe': [(tak_nie, tak_nie_komunikat, [])],
	'dowp_wypadkowe': [(tak_nie, tak_nie_komunikat, [])],
	'doou_data_powstania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dobu_wnosze_emerytalne': [(tak_nie, tak_nie_komunikat, [])],
	'dobu_data_emerytalna': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dobu_wnosze_rentowe': [(tak_nie, tak_nie_komunikat, [])],
	'dobu_data_rentowa': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dobu_wnosze_chorobowe': [(tak_nie, tak_nie_komunikat, [])],
	'dobu_data_chorobowa': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dodu_data_zdrowotnego': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dodu_kwota': [(liczba_dop, liczba_komunikat, [9])],
	'iodz_jestem': [(lista, lista_komunikat, [kody.stopien_pokrewienstwa_lub_powinowactwa])],
	'iodz_pozostaje': [(tak_nie, tak_nie_komunikat, [])],
	'iodz_kod_stopnia': [(lista, lista_komunikat, [kody.stopien_niezdolnosci_do_pracy])],
	'iodz_okres_niezdolnosci': [(okres, okres_komunikat, [])],
	'iodz_kod_zawodu': [(lista, lista_komunikat, [kody.zawod])],
	'iodz_kod_gornika': [(liczba, liczba_komunikat, [6])], # !!!!!!
	'iodz_okres_pracy_gorniczej': [(okres, okres_komunikat, [])],
	'iodz_kod_wyksztalcenia': [(lista, lista_komunikat, [kody.wyksztalcenie])],
	'iodz_kod_szczegolny': [(nazwa, liczba_komunikat, [9])], # !!!!!!
	'iodz_okres_szczegolny': [(okres, okres_komunikat, [])],
	'dokc_kod_kasy': [(lista, lista_komunikat, [kody.kasa_chorych])],
	'dokc_nazwa_kasy': [(nazwa, nazwa_komunikat, [23])],
	'dokc_data_zawarcia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'azmo_kod': [(liczba, kod_komunikat, [5])], 
	'azmo_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'azmo_gmina': [(nazwa, nazwa_komunikat, [26])],
	'azmo_ulica': [(nazwa, nazwa_komunikat, [30])],
	'azmo_dom': [(nazwa, numer_komunikat, [7])],
	'azmo_lokal': [(nazwa, numer_komunikat, [7])],
	'azmo_telefon': [(telefon_12, numer_komunikat, [])],
	'azmo_fax': [(telefon_12, numer_komunikat, [])],
	'adza_kod': [(liczba, kod_komunikat, [5])], 
	'adza_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'adza_gmina': [(nazwa, nazwa_komunikat, [26])],
	'adza_ulica': [(nazwa, nazwa_komunikat, [30])],
	'adza_dom': [(nazwa, numer_komunikat, [7])],
	'adza_lokal': [(nazwa, numer_komunikat, [7])],
	'adza_telefon': [(telefon_12, numer_komunikat, [])],
	'adkb_kod': [(liczba, kod_komunikat, [5])], 
	'adkb_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'adkb_ulica': [(nazwa, nazwa_komunikat, [30])],
	'adkb_dom': [(nazwa, numer_komunikat, [7])],
	'adkb_lokal': [(nazwa, numer_komunikat, [7])],
	'adkb_skrytka': [(nazwa, numer_komunikat, [5])],
	'adkb_telefon': [(telefon_12, numer_komunikat, [])],
	'adkb_fax': [(telefon_12, numer_komunikat, [])],
	'adkb_email': [(adres_30, adres_komunikat, [])],
	'opl1_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])]
	}


tablica_wer['zuszwua'] = {
	'dozwua_wyr_z_us_i_uz': [(tak_nie, tak_nie_komunikat, [])],
	'dozwua_wyr_z_us': [(tak_nie, tak_nie_komunikat, [])],
	'dozwua_wyr_z_uz': [(tak_nie, tak_nie_komunikat, [])],
	'dozwua_korekta_wyr_z_us': [(tak_nie, tak_nie_komunikat, [])],
	'dozwua_data_nadania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dozwua_numer_nalepki_r': [(dowolne, dlugosc_komunikat, [6])],
	'dipl_nip': [(nip, nip_komunikat, [])],
	'dipl_regon': [(regon, regon_komunikat, [])],
	'dipl_pesel': [(pesel, pesel_komunikat, [])],
	'dipl_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dipl_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dipl_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dipl_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dipl_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dipl_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dipl_pesel'])],
	'dipl_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'dau_pesel': [(pesel, pesel_komunikat, [])],
	'dau_nip': [(nip, nip_komunikat, [])],
	'dau_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dau_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dau_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dau_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dau_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dau_pesel'])],
	'dau_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dau_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dau_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'wubd_kod': [(lista_3, lista_komunikat, 
			[kody.tytul_ubezpieczenia_podmiot,
			4, kody.tytul_ubezpieczenia_prawo,
			5, kody.tytul_ubezpieczenia_stopien])],
	'wubd_emerytalne': [(tak_nie, tak_nie_komunikat, [])],
	'wubd_data_emerytalna': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'wubd_kod_przyczyny_emerytalny': [(lista, lista_komunikat, [kody.przyczyna_wyrejestrowania_ubezpieczonego])],
	'wubd_rentowe': [(tak_nie, tak_nie_komunikat, [])],
	'wubd_data_rentowa': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'wubd_kod_przyczyny_rentowy': [(lista, lista_komunikat, [kody.przyczyna_wyrejestrowania_ubezpieczonego])],
	'wubd_chorobowe': [(tak_nie, tak_nie_komunikat, [])],
	'wubd_data_chorobowa': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'wubd_kod_przyczyny_chorobowy': [(lista, lista_komunikat, [kody.przyczyna_wyrejestrowania_ubezpieczonego])],
	'wubd_wypadkowe': [(tak_nie, tak_nie_komunikat, [])],
	'wubd_data_wypadkowa': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'wubd_kod_przyczyny_wypadkowy': [(lista, lista_komunikat, [kody.przyczyna_wyrejestrowania_ubezpieczonego])],
	'wubd_zdrowotne': [(tak_nie, tak_nie_komunikat, [])],
	'wubd_data_zdrowotna': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'wubd_kod_przyczyny_zdrowotny': [(lista, lista_komunikat, [kody.przyczyna_wyrejestrowania_ubezpieczonego])],
	'wnou_kod': [(lista_3, lista_komunikat, 
			[kody.tytul_ubezpieczenia_podmiot,
			4, kody.tytul_ubezpieczenia_prawo,
			5, kody.tytul_ubezpieczenia_stopien])],
	'wnou_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'opl1_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])]
	}


tablica_wer['zuszza'] = {
	'dozza_zgloszenie_do_uz': [(tak_nie, tak_nie_komunikat, [])],
	'dozza_zgloszenie_korekty': [(lista, lista_komunikat, [kody.zgloszenie_zmiany_korekta_danych])],
	'dozza_data_nadania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dozza_numer_nalepki_r': [(dowolne, dlugosc_komunikat, [6])],
	'dipl_nip': [(nip, nip_komunikat, [])],
	'dipl_regon': [(regon, regon_komunikat, [])],
	'dipl_pesel': [(pesel, pesel_komunikat, [])],
	'dipl_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dipl_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dipl_nazwa_skrocona': [(nazwa, nazwa_komunikat, [31])],
	'dipl_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dipl_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dipl_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dipl_pesel'])],
	'dipl_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dipl_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'dau_pesel': [(pesel, pesel_komunikat, [])],
	'dau_nip': [(nip, nip_komunikat, [])],
	'dau_rodzaj_dokumentu': [(lista, lista_komunikat, [kody.rodzaj_dokumentu])],
	'dau_seria_numer': [(seria_numer_9, seria_numer_komunikat, [])],
	'dau_nazwisko': [(nazwisko_31, nazwisko_komunikat, [])],
	'dau_imie_pierwsze': [(imie_22, imie_komunikat, [])],
	'dau_data_urodzenia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, []),
			(urodziny, urodziny_komunikat, ['dau_pesel'])],
	'dau_id_systemowy': [(dowolne, dlugosc_komunikat, [16])],
	'dau_numer_wer_danych_id': [(dowolne, dlugosc_komunikat, [16])],
	'dau_poziom_ufnosci_id': [(dowolne, dlugosc_komunikat, [2])],
	'deoz_imie_drugie': [(imie_22, imie_komunikat, [])],
	'deoz_nazwisko_rodowe': [(nazwisko_31, nazwisko_komunikat, [])],
	'deoz_obywatelstwo': [(obywatelstwo_22, obywatelstwo_komunikat, [])],
	'deoz_plec': [(lista, lista_komunikat, [kody.oznaczenie_plci]),
			(plec, plec_komunikat, ['dau_pesel'])],
	'deoz_czy_ma_karte': [(tak_nie, tak_nie_komunikat, [])],
	'deoz_czy_czasowo': [(tak_nie, tak_nie_komunikat, [])],
	'tyubz_kod': [(lista_3, lista_komunikat, 
			[kody.tytul_ubezpieczenia_podmiot,
			4, kody.tytul_ubezpieczenia_prawo,
			5, kody.tytul_ubezpieczenia_stopien])],
	'doou_data_powstania': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dodu_data_zdrowotnego': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'dodu_kwota': [(liczba_dop, liczba_komunikat, [9])],
	'dokc_kod_kasy': [(lista, lista_komunikat, [kody.kasa_chorych])],
	'dokc_nazwa_kasy': [(nazwa, nazwa_komunikat, [23])],
	'dokc_data_zawarcia': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])],
	'azmo_kod': [(liczba, kod_komunikat, [5])], 
	'azmo_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'azmo_gmina': [(nazwa, nazwa_komunikat, [26])],
	'azmo_ulica': [(nazwa, nazwa_komunikat, [30])],
	'azmo_dom': [(nazwa, numer_komunikat, [7])],
	'azmo_lokal': [(nazwa, numer_komunikat, [7])],
	'azmo_telefon': [(telefon_12, numer_komunikat, [])],
	'azmo_fax': [(telefon_12, numer_komunikat, [])],
	'adza_kod': [(liczba, kod_komunikat, [5])], 
	'adza_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'adza_gmina': [(nazwa, nazwa_komunikat, [26])],
	'adza_ulica': [(nazwa, nazwa_komunikat, [30])],
	'adza_dom': [(nazwa, numer_komunikat, [7])],
	'adza_lokal': [(nazwa, numer_komunikat, [7])],
	'adza_telefon': [(telefon_12, numer_komunikat, [])],
	'adkb_kod': [(liczba, kod_komunikat, [5])], 
	'adkb_miejscowosc': [(nazwa, nazwa_komunikat, [26])],
	'adkb_ulica': [(nazwa, nazwa_komunikat, [30])],
	'adkb_dom': [(nazwa, numer_komunikat, [7])],
	'adkb_lokal': [(nazwa, numer_komunikat, [7])],
	'adkb_skrytka': [(nazwa, numer_komunikat, [5])],
	'adkb_telefon': [(telefon_12, numer_komunikat, [])],
	'adkb_fax': [(telefon_12, numer_komunikat, [])],
	'adkb_email': [(adres_30, adres_komunikat, [])],
	'opl1_data': [(data_ddmmrrrr, data_ddmmrrrr_komunikat, [])]
	}
